""" 
Freedom page 

This module handle downloading and opening web pages.

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import sys
import os
import errno
import re
import urllib
import tempfile
import subprocess
import tarfile
import shutil
import cStringIO
import webbrowser
import logging
import time
import hashlib

from freedom import dispatch
from freedom import config
from freedom import event
from freedom import response

class error(Exception):
    """ Page error """

class downloader(object):

    def __init__(self, next):
        self._next = next
        self._queue = dispatch.queue(self)
        self._tempdir = None 
    
    ## Running

    def start(self):
        self._queue.start()

    def stop(self):
        self._queue.stop()

    ## Handling requests

    def handle(self, request):
        self._queue.handle(request)

    ## dispatch.queue delegate methods

    def queue_handle(self, request):
        self._download(request)
        self._next.handle(request)
   
    ## Private

    def _download(self, req):
        logging.info('page.downloader fetching url: %r', req['url'])
        event.log('server.download', time.time())
        self._tempdir = tempfile.mkdtemp(prefix='freedom.')
        try:
            status = self._run_wget(req)
            # XXX Check status and handle critical failures
            logging.debug('page.downloader wget exit status: %d', status)
            event.log('server.build', time.time())
            builder = response.builder(req) 
            req['response'] = builder.build(self._tempdir, req['filename'])
        finally:
            if not config.debug:
                shutil.rmtree(self._tempdir)
                self._tempdir = None
     
    def _run_wget(self, req):
        logging.debug('page.downloader running wget')
        start = time.time()
        url_hash = hashlib.md5(req['url']).hexdigest()
        dir_prefix = os.path.join(self._tempdir, url_hash)
        wget = subprocess.Popen([
            self._wget_path(),
            '--execute',
            'robots=off',
            '--no-verbose',
            '--user-agent=Mozila',
            '--page-requisites',
            '--adjust-extension',
            '--span-hosts',
            '--convert-links',
            '--directory-prefix=%s' % dir_prefix,
            '--restrict-file-names=windows,ascii',
            '--ignore-tags=script',
            '--no-directories',
            '--no-host-directories',
            req['url'],
            ],
            stderr=subprocess.PIPE)
        stdout, stderr = wget.communicate() 
        req['filename'] = self._find_filename(stderr)
        logging.debug('clock: download: %.2fs', time.time() - start)
        return wget.returncode

    def _find_filename(self, output):
        """ Find the filename of the page after wget manipulation
        
        wget follow redirects and then replace invalid characters in the
        filename. We must get the actual filename stored, which the client will
        open.
        
        WARNING: depends on the format of wget log in the --no-verbose mode.
        """
        # The first line report the main url
        line = output.splitlines()[0]
        # wget normalize "\\" to "/" in paths
        prefix = self._tempdir.replace("\\", "/")
        pattern = r'-> "%s/(.+)"' % (re.escape(prefix),)
        match = re.search(pattern, line)
        if match is None:
            raise error('no filename in wget output: %r' % line)
        return match.group(1)

    def _wget_path(self):
        if hasattr(sys, 'frozen'): # Built with py2exe
            executable = sys.executable
        else:
            executable = sys.argv[0]
        install_dir = os.path.dirname(executable)
        path = os.path.join(install_dir, sys.platform, 'wget')
        logging.debug('page.downloader using wget at: %r', path)
        return path

class opener(object):

    if os.name == 'nt':
        cache_dir = os.path.expanduser(r'~\Application Data\freedom\cache')
    else: # posix
        cache_dir = os.path.expanduser('~/.freedom/cache')

    def __init__(self, next=None):
        self._next = next
        self._queue = dispatch.queue(self)
    
    ## Running

    def start(self):
        self._queue.start()

    def stop(self):
        self._queue.stop()

    ## Handling requests

    def handle(self, response):
        self._queue.handle(response)

    ## dispatch.queue delegate methods

    def queue_handle(self, response):
        logging.info('page.opener opening page: %r', response['url'])
        event.log('client.open', time.time())
        start = time.time()
        self._make_cache_dir()
        self._store_response(response)
        url = self._file_url(response['filename'])
        self._open_browser(url)
        event.log('client.finished', time.time())
        logging.debug('clock: open_page: %.2fs', time.time() - start)
        if self._next:
            self._next.handle(response)

    ## Private

    def _store_response(self, res):
        logging.debug('page.opener storing response')
        start = time.time()
        f = cStringIO.StringIO(res['data'])
        try:
            tar = tarfile.open(None, 'r', fileobj=f)
        except tarfile.ReadError, e:
            raise error('Invalid response format: %s' % e)
        try:
            # Don't hide extraction errors
            tar.errorlevel== 1
            tar.extractall(self.cache_dir)
        except (OSError, IOError), e:
            raise error('Error storing response: %s' % e)
        finally:
            tar.close()
            f.close()
        logging.debug('clock: open_page.store_response: %.2fs', time.time() - start)

    def _file_url(self, filename):
        path = urllib.quote(filename)
        prefix = urllib.pathname2url(self.cache_dir)
        return 'file://' + prefix + '/' + path

    def _open_browser(self, url):
        logging.debug('page.opener opening url %r', url)
        webbrowser.open(url)

    def _make_cache_dir(self):
        try:
            os.makedirs(self.cache_dir)
        except OSError, e:
            if e.errno != errno.EEXIST:
                raise

