""" 
Freedom response 

This module contains builder classes for building response in the server.

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import os
import errno
import tarfile
import cStringIO
import uuid
import logging
import mimetypes
import urllib
import time

from email.message import Message
from email.encoders import encode_base64
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage

from freedom import BeautifulSoup
from freedom import dtd
from freedom import config

def builder(req):
    """ Returns the builder suitable for this request """
    if req.get('attach'):
        return tar_builder()
    else:
        return multipart_builder()

class tar_builder(object):
    """ Builds an email message with a tarball attachment """     
    
    def build(self, pagedir, filename):
        logging.info('response.tar_builder building reply')
        start = time.time()
        path = os.path.join(pagedir, filename)
        page = parse(path)
        sanitize(page)
        with open(path, 'wb') as f:
            f.write(str(page))
        file = cStringIO.StringIO()
        tar = tarfile.open(None, 'w:gz', fileobj=file)
        try:
            tar.add(pagedir, '')
        finally:
            tar.close()
        data = file.getvalue()
        msg = Message()
        msg['content-type'] = 'application/octet-stream'
        msg['content-disposition'] = 'attachment; filename=page.bin'
        msg['x-freedom-filename'] = filename
        msg.set_payload(data)
        encode_base64(msg)
        logging.debug('clock: build_reply: %.2fs', time.time() - start)
        return msg

class multipart_builder(object):
    
    def __init__(self):
        self.seen = None
        self.parts = None

    def build(self, pagedir, filename):
        logging.info('response.multipart_builder building reply')
        start = time.time()
        self.seen = {}
        self.parts = []
        path = os.path.join(pagedir, filename)
        page = parse(path)
        sanitize(page)
        base = os.path.dirname(filename)
        for img in page.findAll('img', src=True):
            src = img['src']
            path = self.normalize_filename(pagedir, base, src)
            if path in self.seen:
                img['src'] = 'cid:%s' % self.seen[path]
            else:
                self.add_image(img, path)
        for link in page.findAll('link', rel='stylesheet'):
            if self.is_screen_stylesheet(link):
                href = link.get('href')
                path = self.normalize_filename(pagedir, base, href)
                self.add_stylesheet(page, link, path)
            else:
                link.extract()
        msg = MIMEMultipart(_subtype='related')
        data = str(page)
        if config.debug:
            self.write_modified_page(pagedir, filename, data)
        part = MIMEText(data, _subtype='html', _charset='utf-8')
        msg.attach(part)
        for part in self.parts:
            msg.attach(part)
        logging.debug('clock: build_reply: %.2fs', time.time() - start)
        return msg
    
    def normalize_filename(self, pagedir, base, filename):
        # wget quote names inside attributes, but we must use the real filename.
        filename = urllib.unquote(filename)
        if filename.startswith(os.sep):
            # We see this sometimes in wikipedia pages - looks like wget bug
            filename = filename[1:]
            host = os.path.split(base)[0]
            path = os.path.join(pagedir, host, filename)
        else:
            path = os.path.join(pagedir, base, filename)
        return os.path.normpath(path)

    def add_image(self, img, path):
        try:
            with open(path, 'rb') as f:
                data = f.read()
        except IOError, e:
            if e.errno != errno.ENOENT:
                raise
            # We don't want to access this url
            logging.info('no such file: %r', path)
            img['src'] = ''
        else:
            cid = str(uuid.uuid4())
            self.seen[path] = cid
            img['src'] = 'cid:%s' % cid
            subtype = self.guess_subtype(path)
            part = MIMEImage(data, _subtype=subtype)
            part['content-id'] = '<%s>' % cid
            self.parts.append(part)

    def is_screen_stylesheet(self, link):
        media = link.get('media', 'all')                            # -> 'Print, Screen '
        media = set([v.strip().lower() for v in media.split(',')])  # -> set(['print', 'screen'])
        media = media.intersection(('screen', 'all'))               # -> set(['screen'])
        return len(media) != 0                                      # -> True

    def add_stylesheet(self, page, link, path):
        try:
            with open(path, 'rb') as f:
                data = f.read()
        except IOError, e:
            if e.errno != errno.ENOENT:
                raise
            # We don't want to access this url
            logging.info('no such file: %r', path)
            link.extract()
        else:
            style = BeautifulSoup.Tag(page, 'style', [('type', 'text/css')])
            if page.isxml:
                # In XHTML, style elements are declared as having #PCDATA
                # content, and must not contain > or & characters.
                data = data.replace('&', '&amp;') # Must be first
                data = data.replace('<', '&lt;') 
            style.insert(0, data)
            link.replaceWith(style)

    def guess_subtype(self, filename):
        type, encoding = mimetypes.guess_type(filename, strict=False)
        if type and '/' in type:
            return type.split('/')[1]
        return None

    def write_modified_page(self, pagedir, filename, data):
        basename, ext = os.path.splitext(filename)
        path = os.path.join(pagedir, basename + '-modified' + ext)
        with open(path, 'wb') as f:
            f.write(data)

def parse(path):
    logging.debug('response parsing page')
    start = time.time()
    with open(path, 'rb') as f:
        data = f.read()
    doctype = dtd.parse(data)
    # BeautifulSoup documentation recommends to parse XML with
    # BeautifulStoneSoup, but testing in the field show that it breks pages
    # layout, while BeautifulSoup handles XHTML just fine.
    page = BeautifulSoup.BeautifulSoup(data)
    page.isxml = doctype.isxml()
    logging.debug('clock: build_reply.parse_page: %.2fs', time.time() - start)
    return page

def sanitize(page):
    # Scripts may try to access restricted URLs, which may lead to detection
    # of the browsing.
    for script in page.findAll('script'):
        script.extract()

