"""
Freedom XMPP

XMPP connections used to send and receive requests.

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import logging
import time

from freedom import xmpplib
from freedom import dispatch
from freedom import event
from freedom import request

class connection_error(Exception):
    pass

class connection(object):
    
    # Time to wait becore reconnecting to the server
    min_reconnect_timeout = 1.0
    max_reconnect_timeout = 120.0

    def __init__(self, host, user, password):
        self._host = host
        self._user = user
        self._password = password
        self._queue = dispatch.queue(self)
        self._jid = None
        self._client = None
        self._reconnect_timeout = self.min_reconnect_timeout

    def start(self):
        self._queue.start()

    def stop(self):
        self._queue.stop()

    def handle(self, msg):
        self._queue.handle(msg)

    def queue_start(self):
        self._jid = xmpplib.JID(self._user)
        self.handle('wakeup')

    def _connect(self):
        self._client = xmpplib.Client(self._jid.getDomain(), debug=[])
        self._client.RegisterDisconnectHandler(self._handle_disconnect);
        logging.info('xmpp.listener connecting to %r', self._host)
        if not self._client.connect(self._host):
            raise connection_error('xmpp.listener cannot connect')
        logging.info('xmpp.listener logging in as: %r', self._user)
        if not self._client.auth(self._jid.getNode(), self._password):
            raise connection_error('authtication error')
        logging.debug('xmpp.listener sending presence')
        self._client.sendInitPresence(requestRoster=1)
        self._reconnect_timeout = self.min_reconnect_timeout
    
    def _disconnect(self):
        if self._client:
            logging.info('xmpp.listener disconnecting')
            # This xmmp library sucks - we have to poke into client internals to
            # close things properly.
            for name in ['ComponentBind', 'Bind', 'NonSASL', 'SASL', 'TLS', 
                    'Dispatcher', 'HTTPPROXYsocket', 'TCPsocket']:
                plugin = getattr(self._client, name, None)
                if plugin:
                    plugin.PlugOut()
            self._client = None

    def _wait_before_reconnect(self):
        logging.info('xmpp.listener reconnecting in %.0f seconds...', self._reconnect_timeout)
        time.sleep(self._reconnect_timeout)
        # Be polite to the server
        if self._reconnect_timeout < self.max_reconnect_timeout:
            self._reconnect_timeout *= 2
    
    def _handle_disconnect(self):
        logging.info('xmpp.listener disconnected')
        raise connection_error('disconnected')

class listener(connection):

    def __init__(self, host, user, password, next):
        connection.__init__(self, host, user, password)
        self._next = next

    def queue_handle(self, msg):
        try:
            if self._client is None:
                self._connect()
                self._register_handlers()
            self._client.Process(300)
        except connection_error, e:
            self._disconnect()
            self._wait_before_reconnect()
        except Exception:
            logging.exception('xmpp.listener error')

        self.handle(msg)

    def _register_handlers(self):
        self._client.RegisterHandler('message', self._handle_message)
        self._client.RegisterHandler('presence', self._handle_presence)

    def _handle_message(self, con, ev):
        type = ev.getType()
        if type not in ['message', 'chat', None]:
            logging.debug('xmpp.listner dropping event: not a message')
            return
        body = ev.getBody().strip()
        try:
            req = request.parse(body)
        except request.error, e:
            logging.debug('xmpp.listener: %s', e)
            return
        event.log('server.receive', time.time())
        # XXX Can this fail?
        sender = ev.getFrom().getStripped()
        req['from'] = sender
        logging.info('xmpp.listener received request %r', req)
        self._next.handle(req)

    def _handle_presence(self, con, ev):
        type = ev.getType()
        if type == 'subscribe':
            subscriber = ev.getFrom()
            logging.info('xmpp.listener accepting subscriber %r', subscriber)
            self._client.sendPresence(jid=subscriber, typ='subscribed')

class tester(connection):
    """ Initiate a request, wait for the resposne, log it and make new request """

    def __init__(self, host, user, password, server, url, max_requests=None):
        connection.__init__(self, host, user, password)
        self._server = server
        self._url = url
        self._max_requests = max_requests
        self._waiting = False
        self._count = 0

    def queue_handle(self, msg):
        if self._waiting:
            self._waiting = False
            logging.info('xmpp.tester received response %d: %r', self._count, msg['url'])
            event.end('tester.finished', time.time())
        if self._count == self._max_requests:
            logging.info('xmpp.tester finished')
            return
        try:
            if self._client is None:
                self._connect()
            event.start('tester.send', time.time())
            msg = xmpplib.Message(self._server, self._url)
            self._client.send(msg)
            self._waiting = True
            self._count += 1
            logging.info('xmpp.tester sent request %d: %r', self._count, self._url)
            # Drain messags from server
            self._client.Process(1)
        except connection_error, e:
            self._disconnect()
            self._wait_before_reconnect()
        except Exception:
            logging.exception('xmpp.tester error')

