from freedom import dtd

def test_parse(doctype, public_id, uri, data):
    dt = dtd.parse(data)
    assert dt.doctype == doctype
    assert dt.public_id == public_id
    assert dt.uri == uri

test_parse.tests = [
    # Tag soup
    dict(doctype=None,
        public_id=None, 
        uri=None,
        data='<h1>Tag Soup</h1>'),
    # Public id with upper case type
    dict(doctype='html',
        public_id='-//W3C//DTD HTML 4.01//EN',
        uri='http://www.w3.org/TR/html4/strict.dtd',
        data='''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
           "http://www.w3.org/TR/html4/strict.dtd">'''),
    # Public id with lower case type
    dict(doctype='html',
        public_id='-//W3C//DTD XHTML 1.0 Strict//EN',
        uri='http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd',
        data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
           "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'''),
    # XHTML with xml decleration - not supported by all browsers
    dict(doctype='html',
        public_id='-//W3C//DTD XHTML 1.0 Strict//EN',
        uri='http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd',
        data='''<?xml version="1.0" standalone="no"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'''),
    # Public id with lower case public
    dict(doctype='html',
        public_id='-//W3C//DTD HTML 4.01//EN',
        uri='http://www.w3.org/TR/html4/strict.dtd',
        data='''<!DOCTYPE html public "-//W3C//DTD HTML 4.01//EN"
           "http://www.w3.org/TR/html4/strict.dtd">'''),
    # Public id with single quotes
    dict(doctype='html',
        public_id='-//W3C//DTD XHTML 1.0 Strict//EN',
        uri='http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd', 
        data='''<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN'
           'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>'''),
    # System uri - not used for html, for completeness
    dict(doctype='html', 
        public_id=None,
        uri='http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd', 
        data='''<!DOCTYPE html SYSTEM "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'''),
    # System uri - with lower case system 
    dict(doctype='html',
        public_id=None,
        uri='http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd', 
        data='''<!DOCTYPE html system "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'''),
    # Without public id or system uri - html5 dtd
    dict(doctype='html',
        public_id=None,
        uri=None, 
        data='''<!DOCTYPE html>'''),
    # Internal dtd - not used in html, for completeness
    dict(doctype='html',
        public_id=None,
        uri=None, 
        data='''<!DOCTYPE html []>'''),
]

def test_isxml(data):
    dt = dtd.parse(data)
    assert dt.isxml()

test_isxml.tests = [
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" 
            "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML Basic 1.1//EN"
            "http://www.w3.org/TR/xhtml-basic/xhtml-basic11.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
           "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
           "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN"
           "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC
            "-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN"
                "http://www.w3.org/2002/04/xhtml-math-svg/xhtml-math-svg.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML Basic 1.0//EN"
            "http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd">'''),
]

def test_notxml(data):
    dt = dtd.parse(data)
    assert not dt.isxml()

test_notxml.tests = [
    dict(data='''<!DOCTYPE HTML>'''),
    dict(data='''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
           "http://www.w3.org/TR/html4/strict.dtd">'''),
    dict(data='''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
           "http://www.w3.org/TR/html4/loose.dtd">'''),
    dict(data='''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"
           "http://www.w3.org/TR/html4/frameset.dtd">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">'''),
    dict(data='''<!DOCTYPE html PUBLIC "-//IETF//DTD HTML 2.0//EN">'''),
    dict(data='''<html><head><title>It works</title></head><body></body></html>'''),
    dict(data='''<h1>It works in my browser!</h1>'''),
    dict(data='''It works in my browser!'''),
]

def test_real_pages(filename, doctype, public_id, isxml):
    with open(filename) as f:
        data = f.read()
    dt = dtd.parse(data)
    assert dt.doctype == doctype
    assert dt.public_id == public_id
    assert dt.isxml() == isxml

test_real_pages.tests = [
    dict(filename='test/html/amazon.html', 
        doctype='html',
        public_id='-//W3C//DTD HTML 4.01 Transitional//EN',
        isxml=False),
    dict(filename='test/html/google-search.html',
        doctype='html',
        public_id=None,
        isxml=False),
    dict(filename='test/html/il.msn.com.html',
        doctype='html',
        public_id='-//W3C//DTD XHTML 1.0 Strict//EN',
        isxml=True),
    dict(filename='test/html/qq.html',
        doctype='html',
        public_id='-//W3C//DTD XHTML 1.0 Transitional//EN',
        isxml=True),
    dict(filename='test/html/wikipedia.html', 
        doctype='html', 
        public_id='-//W3C//DTD XHTML 1.0 Transitional//EN',
        isxml=True),
    dict(filename='test/html/wordpress.html',
        doctype='html',
        public_id='-//W3C//DTD XHTML 1.0 Transitional//EN',
        isxml=True),
    dict(filename='test/html/yahoo.co.jp.html', 
        doctype='html',
        public_id='-//W3C//DTD HTML 4.01 Transitional//EN',
        isxml=False),
    dict(filename='test/html/yahoo.com.html', 
        doctype='html',
        public_id=None,
        isxml=False),
    dict(filename='test/html/ynet.html',
        doctype=None,
        public_id=None,
        isxml=False),
    dict(filename='test/html/youtube.html',
        doctype='html',
        public_id=None,
        isxml=False),
]

def pytest_generate_tests(metafunc):
    for funcargs in getattr(metafunc.function, 'tests', ()):
        metafunc.addcall(funcargs=funcargs)

